# Copyright 2018-2019 The Wazo Authors  (see the AUTHORS file)
# SPDX-License-Identifier: GPL-3.0-or-later

import logging
import jwt as jwt_module

from requests.exceptions import HTTPError
from wazo_auth import BaseMetadata
from wazo_auth_client import Client as AuthClient

logger = logging.getLogger()


class NestboxJWT(BaseMetadata):

    def __init__(self):
        self._jwt = None

    def load(self, dependencies):
        super().load(dependencies)
        self._nestbox_config = self._extract_config(dependencies['config'])
        self._user_service = dependencies['user_service']

    def _extract_config(self, config):
        try:
            config['nestbox']['auth']['username'] = config['nestbox']['auth'].pop('service_id')
            config['nestbox']['auth']['password'] = config['nestbox']['auth'].pop('service_key')
        except KeyError:
            raise Exception('Missing nestbox authentication configuration')

        return config['nestbox']

    def get_token_metadata(self, login, args):
        metadata = super().get_token_metadata(login, args)
        metadata['xivo_user_uuid'] = self._get_user_uuid(login)

        if self._is_valid(self._jwt):
            metadata['jwt'] = self._jwt
            return metadata

        logger.debug('Getting new subscription authorization')
        client = AuthClient(**self._nestbox_config['auth'])
        try:
            nestbox_token = client.token.new(expiration=10)
        except HTTPError as e:
            logger.error(e)
            return metadata

        try:
            metadata['jwt'] = nestbox_token['metadata']['jwt']
        except KeyError:
            logger.warning("Nestbox token doesn't have an active subscription authorization")
            return metadata

        logger.debug('New subscription authorization fetched')
        self._jwt = metadata['jwt']
        return metadata

    def _is_valid(self, jwt):
        if not jwt:
            return False

        try:
            jwt_module.decode(jwt, options={'verify_signature': False, 'verify_aud': False})
        except jwt_module.ExpiredSignatureError:
            logger.debug('Cached subscription authorization is expired: %s', jwt)
            return False
        except Exception as e:
            logger.error(e)
            return False

        return True

    def get_acl_metadata(self, **kwargs):
        return {}

    def _get_user_uuid(self, username):
        users = self._user_service.list_users(username=username)
        for user in users:
            return user['uuid']
